@@ -32,6 +32,7 @@
 #include <sys/socket.h>
 #include <netinet/in.h> //For sockaddr_in
 
+#include <signal.h> //To catch segfaults
 
 #include "gst-package.h"
 
@@ -47,7 +48,7 @@ namespace Glom
 # local is for Unix domain socket connections only\n\
 local   all         all                               ident sameuser\n\
 # IPv4 local connections:\n\
-host    all         all         127.0.0.1/32          md5\n\
+host    all         all         0.0.0.0/0          md5\n\
 # IPv6 local connections:\n\
 host    all         all         ::1/128               md5\n"
 
@@ -234,23 +235,24 @@ sharedptr<SharedConnection> ConnectionPool::connect()
         bool try_another_port = true;
         while(try_another_port)
         { 
-          const Glib::ustring cnc_string_main = "HOST=" + get_host() + ";USER=" + m_user + ";PASSWORD=" + m_password + ";PORT=" + port;
+//          const Glib::ustring cnc_string_main = "HOST=" + get_host() + ";USER=" + m_user + ";PASSWORD=" + m_password + ";PORT=" + port;
+	  const Glib::ustring cnc_string_main = "HOST=" + get_host() + ";PORT=" + port;
 
           Glib::ustring cnc_string = cnc_string_main;
 
           if(!m_database.empty())
-            cnc_string += (";DATABASE=" + m_database);
+            cnc_string += (";DB_NAME=" + m_database);
           else
-            cnc_string += (";DATABASE=" + default_database);
+            cnc_string += (";DB_NAME=" + default_database);
 
           //std::cout << "debug: connecting: cnc string: " << cnc_string << std::endl;
           std::cout << std::endl << "Glom: trying to connect on port=" << port << std::endl;
 
           //*m_refGdaConnection = m_GdaClient->open_connection(m_GdaDataSourceInfo.get_name(), m_GdaDataSourceInfo.get_username(), m_GdaDataSourceInfo.get_password() );
           //m_refGdaConnection.clear(); //Make sure any previous connection is really forgotten.
-          m_refGdaConnection = m_GdaClient->open_connection_from_string("PostgreSQL", cnc_string);
-          if(m_refGdaConnection)
+          try
           {
+            m_refGdaConnection = m_GdaClient->open_connection_from_string("PostgreSQL", cnc_string, m_user, m_password);
             //g_warning("ConnectionPool: connection opened");
 
             //Remember what port is working:
@@ -261,7 +263,7 @@ sharedptr<SharedConnection> ConnectionPool::connect()
               m_pFieldTypes = new FieldTypes(m_refGdaConnection);  
 
             //Enforce ISO formats in the communication:
-            m_refGdaConnection->execute_single_command("SET DATESTYLE = 'ISO'");  
+            m_refGdaConnection->execute_non_select_command("SET DATESTYLE = 'ISO'");  
 
             //Open the database, if one has been specified:
             /* This does not seem to work in libgda's postgres provider, so we specify it in the cnc_string instead:
@@ -271,11 +273,11 @@ sharedptr<SharedConnection> ConnectionPool::connect()
             */
 
             //Get postgres version:
-            Glib::RefPtr<Gnome::Gda::DataModel> data_model = m_refGdaConnection->execute_single_command("SELECT version()");
+            Glib::RefPtr<Gnome::Gda::DataModel> data_model = m_refGdaConnection->execute_select_command("SELECT version()");
             if(data_model && data_model->get_n_rows() && data_model->get_n_columns())
             {
               Gnome::Gda::Value value = data_model->get_value_at(0, 0);
-              if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_STRING)
+              if(value.get_value_type() == G_TYPE_STRING)
               {
                 const Glib::ustring version_text = value.get_string();
 
@@ -295,9 +297,9 @@ sharedptr<SharedConnection> ConnectionPool::connect()
 
             return connect(); //Call this method recursively. This time m_refGdaConnection exists.
           }
-          else
+          catch(const Gnome::Gda::ConnectionError& ex)
           {
-            std::cout << "ConnectionPool::connect() Attempt to connect to database failed on port=" << port << ", database=" << m_database << std::endl;
+            std::cout << "ConnectionPool::connect() Attempt to connect to database failed on port=" << port << ", database=" << m_database << ": " << ex.what() << std::endl;
 
             bool bJustDatabaseMissing = false;
             if(!m_database.empty())
@@ -307,21 +309,22 @@ sharedptr<SharedConnection> ConnectionPool::connect()
               //If the connection failed while looking for a database,
               //then try connecting without the database:
               Glib::ustring cnc_string = cnc_string_main;
-              cnc_string += (";DATABASE=" + default_database);
+              cnc_string += (";DB_NAME=" + default_database);
 
               //std::cout << "debug2: connecting: cnc string: " << cnc_string << std::endl;
               std::cout << "Glom: connecting." << std::endl;
 
-              Glib::RefPtr<Gnome::Gda::Connection> gda_connection =  m_GdaClient->open_connection_from_string("PostgreSQL", cnc_string);
-              if(gda_connection) //If we could connect without specifying the database.
-              {
+              try
+	      {
+	        //If we could connect without specifying the database.
+                Glib::RefPtr<Gnome::Gda::Connection> gda_connection = m_GdaClient->open_connection_from_string("PostgreSQL", cnc_string, m_user, m_password);
                 bJustDatabaseMissing = true;
                 connection_to_default_database_possible = true;
                 m_port = port;
               }
-              else
+              catch(const Gnome::Gda::ConnectionError& ex)
               {
-                std::cerr << "    ConnectionPool::connect() connection also failed when not specifying database." << std::endl;
+                std::cerr << "    ConnectionPool::connect() connection also failed when not specifying database: " << ex.what() << std::endl;
               }
             }
 
@@ -378,6 +381,20 @@ void ConnectionPool::set_self_hosted(const std::string& data_uri)
   m_self_hosting_data_uri = data_uri;
 }
 
+void ConnectionPool::create_database(const Glib::ustring& database_name)
+{
+  if(m_GdaClient)
+  {
+    Glib::RefPtr<Gnome::Gda::ServerOperation> op = m_GdaClient->prepare_create_database(database_name, "PostgreSQL");
+    g_assert(op);
+    op->set_value_at("/SERVER_CNX_P/HOST", get_host());
+    op->set_value_at("/SERVER_CNX_P/PORT", m_port);
+    op->set_value_at("/SERVER_CNX_P/ADM_LOGIN", get_user());
+    op->set_value_at("/SERVER_CNX_P/ADM_PASSWORD", get_password());
+    m_GdaClient->perform_create_database(op);
+  }
+}
+
 void ConnectionPool::set_host(const Glib::ustring& value)
 {
   if(value != m_host)
@@ -412,6 +429,11 @@ Glib::ustring ConnectionPool::get_host() const
   return m_host;
 }
 
+/*Glib::ustring ConnectionPool::get_port() const
+{
+  return m_port;
+}*/
+
 Glib::ustring ConnectionPool::get_user() const
 {
   return m_user;
@@ -462,8 +484,8 @@ bool ConnectionPool::handle_error(bool cerr_only)
   {
     Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
 
-    typedef std::list< Glib::RefPtr<Gnome::Gda::Error> > type_list_errors;
-    type_list_errors list_errors = gda_connection->get_errors();
+    typedef std::list< Glib::RefPtr<Gnome::Gda::ConnectionEvent> > type_list_errors;
+    type_list_errors list_errors = gda_connection->get_events();
 
     if(!list_errors.empty())
     {
@@ -503,12 +525,41 @@ float ConnectionPool::get_postgres_server_version()
   return m_postgres_server_version;
 }
 
-bool ConnectionPool::directory_exists(const std::string& uri)
+/*
+bool ConnectionPool::directory_exists_filepath(const std::string& filepath)
+{
+  const std::string uri = Glib::filename_to_uri(filepath);
+
+  return directory_exists_uri(uri);
+}
+*/
+
+bool ConnectionPool::directory_exists_uri(const std::string& uri)
 {
   Glib::RefPtr<Gnome::Vfs::Uri> vfsuri = Gnome::Vfs::Uri::create(uri);
   return vfsuri->uri_exists();
 }
 
+static sighandler_t previous_sig_handler = SIG_DFL; /* Arbitrary default */
+
+/* This is a Linux/Unix signal handler, 
+ * so we can respond to a crash.
+ */
+static void on_linux_signal(int signum)
+{
+  ConnectionPool* connection_pool = ConnectionPool::get_instance();
+  if(!connection_pool)
+    return;
+
+  if(signum == SIGSEGV)
+  {
+    connection_pool->stop_self_hosting();
+	
+    //TODO: How can we let GNOME's crash handler still handle this?
+    exit(1);
+  }
+}
+
 bool ConnectionPool::start_self_hosting()
 {
   if(m_self_hosting_active)
@@ -516,9 +567,10 @@ bool ConnectionPool::start_self_hosting()
 
   const std::string dbdir_uri = m_self_hosting_data_uri;
 
-  if(!(directory_exists(dbdir_uri)))
+  if(!(directory_exists_uri(dbdir_uri)))
   {
-    std::cerr << "ConnectionPool::create_self_hosting(): The directory could not be found: " << dbdir_uri << std::endl;
+    //TODO: Use a return enum or exception so we can tell the user about this:
+    std::cerr << "ConnectionPool::create_self_hosting(): The data directory could not be found: " << dbdir_uri << std::endl;
     return false;
   }
 
@@ -527,8 +579,9 @@ bool ConnectionPool::start_self_hosting()
 
   const std::string dbdir_data = Glib::build_filename(dbdir, "data");
   const std::string dbdir_data_uri = Glib::filename_to_uri(dbdir_data);
-  if(!(directory_exists(dbdir_data_uri)))
+  if(!(directory_exists_uri(dbdir_data_uri)))
   {
+    //TODO: Use a return enum or exception so we can tell the user about this:
     std::cerr << "ConnectionPool::create_self_hosting(): The data sub-directory could not be found." << dbdir_data_uri << std::endl;
     return false;
   }
@@ -537,6 +590,7 @@ bool ConnectionPool::start_self_hosting()
   const int available_port = discover_first_free_port(PORT_POSTGRESQL_SELF_HOSTED_START, PORT_POSTGRESQL_SELF_HOSTED_END);
   if(available_port == 0)
   {
+    //TODO: Use a return enum or exception so we can tell the user about this:
     std::cerr << "ConnectionPool::create_self_hosting(): No port was available between " << PORT_POSTGRESQL_SELF_HOSTED_START << " and " << PORT_POSTGRESQL_SELF_HOSTED_END << std::endl;
     return false;
   }
@@ -580,6 +634,10 @@ bool ConnectionPool::start_self_hosting()
   //Let clients discover this server via avahi:
   avahi_start_publishing();
 
+  //If we crash while self-hosting (unlikely, hopefully) 
+  //then try to stop the postgres instance instead of leaving it running as an orphan.
+  previous_sig_handler = signal(SIGSEGV, &on_linux_signal);
+
   return true;
 }
 
@@ -609,14 +667,26 @@ void ConnectionPool::stop_self_hosting()
   const bool result = Glom::Spawn::execute_command_line_and_wait(command_postgres_stop, _("Stopping Database Server"));
   if(!result)
   {
-    std::cerr << "Error while attempting to stop self-hosting of the database."  << std::endl;
+    std::cerr << "Error while attempting to stop self-hosting of the database. Trying again."  << std::endl;
+
+    //I've seen it fail when running under valgrind, and there are reports of failures in bug #420962.
+    //Maybe it will help to try again:
+    const bool result = Glom::Spawn::execute_command_line_and_wait(command_postgres_stop, _("Stopping Database Server (retrying)"));
+    if(!result)
+    {
+      std::cerr << "Error while attempting (for a second time) to stop self-hosting of the database."  << std::endl;
+    }
   }
 
+  //We don't need the segfault handler anymore:
+  signal(SIGSEGV, previous_sig_handler);
+  previous_sig_handler = SIG_DFL; /* Arbitrary default */
+
   m_self_hosting_active = false;
 }
 
 
-bool ConnectionPool::create_self_hosting()
+bool ConnectionPool::create_self_hosting(Gtk::Window* parent_window)
 {
   if(m_self_hosting_data_uri.empty())
   {
@@ -627,6 +697,17 @@ bool ConnectionPool::create_self_hosting()
   //Get the filepath of the directory that we should create:
   const std::string dbdir_uri = m_self_hosting_data_uri;
   //std::cout << "debug: dbdir_uri=" << dbdir_uri << std::endl;
+
+  if(directory_exists_uri(dbdir_uri))
+  { 
+    if(parent_window)
+    {
+      Utils::show_ok_dialog(_("Directory Already Exists"), _("There is an existing directory with the same name as the directory that should be created for the new database files. You should specify a different filename to use a new directory instead."), *parent_window, Gtk::MESSAGE_ERROR);
+    }
+    
+    return false;
+  }
+
   const std::string dbdir = Glib::filename_from_uri(dbdir_uri);
   //std::cout << "debug: dbdir=" << dbdir << std::endl;
   g_assert(!dbdir.empty());
@@ -638,6 +719,12 @@ bool ConnectionPool::create_self_hosting()
   {
     std::cerr << "Error from g_mkdir_with_parents() while trying to create directory: " << dbdir << std::endl;
     perror("Error from g_mkdir_with_parents");
+
+    if(parent_window)
+    {
+       Utils::show_ok_dialog(_("Could Not Create Directory"), _("There was an error when attempting to create the directory for the new database files."), *parent_window, Gtk::MESSAGE_ERROR);
+    }
+
     return false;
   }
 
@@ -646,6 +733,11 @@ bool ConnectionPool::create_self_hosting()
   mkdir_succeeded = g_mkdir_with_parents(dbdir_config.c_str(), 0770);
   if(mkdir_succeeded == -1)
   {
+    if(parent_window)
+    {
+       Utils::show_ok_dialog(_("Could Not Create Configuration Directory"), _("There was an error when attempting to create the configuration directory for the new database files."), *parent_window, Gtk::MESSAGE_ERROR);
+    }
+
     std::cerr << "Error from g_mkdir_with_parents() while trying to create directory: " << dbdir_config << std::endl;
     perror("Error from g_mkdir_with_parents");
     return false;
@@ -787,7 +879,8 @@ int ConnectionPool::discover_first_free_port(int start_port, int end_port)
 
 bool ConnectionPool::check_postgres_is_available_with_warning()
 {
-  const std::string binpath = Glib::build_filename(POSTGRES_UTILS_PATH, "postmaster");
+  //EXEEXT is defined in the Makefile.am
+  const std::string binpath = Glib::build_filename(POSTGRES_UTILS_PATH, "postmaster" EXEEXT);
   const Glib::ustring uri_binpath = Glib::filename_to_uri(binpath);
   if(Bakery::App_WithDoc::file_exists(uri_binpath))
     return true;
@@ -857,6 +950,31 @@ bool ConnectionPool::install_postgres(Gtk::Window* parent_window)
 #endif
 }
 
+bool ConnectionPool::check_postgres_gda_client_is_available_with_warning()
+{
+  Glib::RefPtr<Gnome::Gda::Client> gda_client = Gnome::Gda::Client::create();
+  if(gda_client)
+  {
+    //Get the list of providers:
+    typedef std::list<Gnome::Gda::ProviderInfo> type_list_of_provider_info;
+    type_list_of_provider_info providers = Gnome::Gda::Config::get_providers();
+
+    //Examine the information about each Provider:
+    for(type_list_of_provider_info::const_iterator iter = providers.begin(); iter != providers.end(); ++iter)
+    { 
+      const Gnome::Gda::ProviderInfo& info = *iter;
+      if (info.get_id() == "PostgreSQL")
+        return true;
+    }
+  }
+
+  /* The Postgres provider was not found, so warn the user: */
+  Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Incomplete Glom Installation")), true /* use_markup */, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true /* modal */);
+  dialog.set_secondary_text(_("Your installation of Glom is not complete, because the PostgreSQL libgda provider is not available on your system. This provider is needed to access Postgres database servers.\n\nPlease report this bug to your vendor, or your system administrator so it can be corrected."));
+  dialog.run();
+  return false;
+}
+
 
 /** Advertise self-hosting via avahi:
  */
