@@ -21,6 +21,8 @@
 #include "glomconversions.h"
 #include <glom/libglom/connectionpool.h>
 #include <glom/libglom/utils.h>
+#include <gdkmm/pixbufloader.h>
+#include <glom/libglom/data_structure/layout/layoutitem_image.h> // For GLOM_IMAGE_FORMAT
 #include <glibmm/i18n.h>
 #include <sstream> //For stringstream
 
@@ -30,13 +32,16 @@
 #include <iomanip>
 
 
-Glib::ustring GlomConversions::format_time(const tm& tm_data)
+namespace Glom
+{
+
+Glib::ustring Conversions::format_time(const tm& tm_data)
 {
   return format_time( tm_data, std::locale("") /* the user's current locale */ ); //Get the current locale.
 }
 
 
-Glib::ustring GlomConversions::format_time(const tm& tm_data, const std::locale& locale, bool iso_format)
+Glib::ustring Conversions::format_time(const tm& tm_data, const std::locale& locale, bool iso_format)
 {
   if(iso_format)
   {
@@ -46,7 +51,7 @@ Glib::ustring GlomConversions::format_time(const tm& tm_data, const std::locale&
     return format_tm(tm_data, locale, _("%X") /* time */);
 }
 
-Glib::ustring GlomConversions::format_date(const tm& tm_data)
+Glib::ustring Conversions::format_date(const tm& tm_data)
 {
   return format_date( tm_data, std::locale("") /* the user's current locale */ ); //Get the current locale.
 }
@@ -59,7 +64,7 @@ static const char* glom_get_locale_date_format()
   return _("%x");
 }
 
-Glib::ustring GlomConversions::format_date(const tm& tm_data, const std::locale& locale, bool iso_format)
+Glib::ustring Conversions::format_date(const tm& tm_data, const std::locale& locale, bool iso_format)
 {
   if(iso_format)
     return format_tm(tm_data, locale, "%F" /* iso-format date */);
@@ -71,7 +76,7 @@ Glib::ustring GlomConversions::format_date(const tm& tm_data, const std::locale&
 }
 
 
-Glib::ustring GlomConversions::format_tm(const tm& tm_data, const std::locale& locale, const char* format)
+Glib::ustring Conversions::format_tm(const tm& tm_data, const std::locale& locale, const char* format)
 {
   //This is based on docs found here:
   //http://www.roguewave.com/support/docs/sourcepro/stdlibref/time-put.html
@@ -124,7 +129,7 @@ Glib::ustring GlomConversions::format_tm(const tm& tm_data, const std::locale& l
   while((bufsize *= 2) <= 65536);
 
   // This error is quite unlikely (unless strftime is buggy).
-  g_warning("GlomConversions::format_time(): maximum size of strftime buffer exceeded, giving up");
+  g_warning("Conversions::format_time(): maximum size of strftime buffer exceeded, giving up");
 
   return Glib::ustring();
   */
@@ -153,29 +158,30 @@ class numpunct_no_thousands_separator: public std::numpunct<char>
 
 } //anonymous namespace
 
-Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value, const NumericFormat& numeric_format)
+Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value, const NumericFormat& numeric_format)
 {
   return get_text_for_gda_value(glom_type, value, std::locale("") /* the user's current locale */, numeric_format); //Get the current locale.
 }
 
-Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value, const std::locale& locale, const NumericFormat& numeric_format, bool iso_format)
+Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value, const std::locale& locale, const NumericFormat& numeric_format, bool iso_format)
 {
-  if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_NULL) //The type can be null for any of the actual field types.
+  if(value.is_null()) //The type can be null for any of the actual field types.
   {
     return "";
   }
 
-  if( (glom_type == Field::TYPE_DATE) && (value.get_value_type() == Gnome::Gda::VALUE_TYPE_DATE))
+  if( (glom_type == Field::TYPE_DATE) && (value.get_value_type() == G_TYPE_DATE))
   {
-    Gnome::Gda::Date gda_date = value.get_date();
+    Glib::Date gda_date = value.get_date();
+    //Gnome::Gda::Date gda_date = value.get_date();
 
     //tm the_c_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
     tm the_c_time;
     memset(&the_c_time, 0, sizeof(the_c_time));
 
-    the_c_time.tm_year = gda_date.year - 1900; //C years start are the AD year - 1900. So, 01 is 1901.
-    the_c_time.tm_mon = gda_date.month - 1; //C months start at 0.
-    the_c_time.tm_mday = gda_date.day; //starts at 1
+    the_c_time.tm_year = gda_date.get_year() - 1900; //C years start are the AD year - 1900. So, 01 is 1901.
+    the_c_time.tm_mon = gda_date.get_month() - 1; //C months start at 0.
+    the_c_time.tm_mday = gda_date.get_day(); //starts at 1
 
     return format_date(the_c_time, locale, iso_format);
 
@@ -185,7 +191,7 @@ Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glo
     return date.format_string("%x"); //%x means "is replaced by the locale's appropriate date representation".
     */
   }
-  else if((glom_type == Field::TYPE_TIME) && (value.get_value_type() == Gnome::Gda::VALUE_TYPE_TIME))
+  else if((glom_type == Field::TYPE_TIME) && (value.get_value_type() == GDA_TYPE_TIME))
   {
     Gnome::Gda::Time gda_time = value.get_time();
 
@@ -199,7 +205,7 @@ Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glo
 
     return format_time(the_c_time, locale, iso_format);
   }
-  else if( (glom_type == Field::TYPE_NUMERIC) && (value.get_value_type() == Gnome::Gda::VALUE_TYPE_NUMERIC))
+  else if( (glom_type == Field::TYPE_NUMERIC) && (value.get_value_type() == GDA_TYPE_NUMERIC))
   {
     const GdaNumeric* gda_numeric = value.get_numeric();
     std::string text_in_c_locale;
@@ -258,9 +264,9 @@ Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glo
   }
 }
 
-Gnome::Gda::Value GlomConversions::parse_value(double number)
+Gnome::Gda::Value Conversions::parse_value(double number)
 {
-  //This is just a way to get a NUMERIC Gda::Value from a numeric type:
+  //This is just a way to get a NUMERIC Gnome::Gda::Value from a numeric type:
   //Try to parse the inputted number, according to the current locale.
 
   GdaNumeric gda_numeric = {0, 0, 0};
@@ -282,13 +288,13 @@ Gnome::Gda::Value GlomConversions::parse_value(double number)
   return Gnome::Gda::Value(&gda_numeric);
 }
 
-Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type, const Glib::ustring& text, bool& success, bool iso_format)
+Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, const Glib::ustring& text, bool& success, bool iso_format)
 {
   NumericFormat ignore_format;
   return parse_value(glom_type, text, ignore_format, success, iso_format);
 }
 
-Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type, const Glib::ustring& text, const NumericFormat& numeric_format, bool& success, bool iso_format)
+Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, const Glib::ustring& text, const NumericFormat& numeric_format, bool& success, bool iso_format)
 {
   std::locale the_locale = (iso_format ? std::locale::classic() :  std::locale("") /* The user's current locale */);
 
@@ -308,10 +314,8 @@ Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type,
   {
     tm the_c_time = parse_date(text, the_locale, success);
 
-    Gnome::Gda::Date gda_date = {0, 0, 0};
-    gda_date.year = the_c_time.tm_year + 1900; //The C time starts at 1900.
-    gda_date.month = the_c_time.tm_mon + 1; //The C month starts at 0.
-    gda_date.day = the_c_time.tm_mday; //THe C mday starts at 1.
+    // The C time starts at 1900 and the C month starts at 0.
+    Glib::Date gda_date(the_c_time.tm_mday, static_cast<Glib::Date::Month>(the_c_time.tm_mon + 1), the_c_time.tm_year + 1900);
 
     return Gnome::Gda::Value(gda_date);
   }
@@ -334,7 +338,7 @@ Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type,
   }
   else if(glom_type == Field::TYPE_NUMERIC)
   {
-    Glib::ustring text_to_parse = GlomUtils::trim_whitespace(text);
+    Glib::ustring text_to_parse = Utils::trim_whitespace(text);
 
     if(!(numeric_format.m_currency_symbol.empty()))
     {
@@ -343,7 +347,7 @@ Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type,
       if(text_to_parse.substr(0, numeric_format.m_currency_symbol.size()) == numeric_format.m_currency_symbol)
       {
         text_to_parse = text_to_parse.substr(numeric_format.m_currency_symbol.size());
-        text_to_parse = GlomUtils::trim_whitespace(text_to_parse); //remove any whitespace between the currency symbol and the number.
+        text_to_parse = Utils::trim_whitespace(text_to_parse); //remove any whitespace between the currency symbol and the number.
       }
     }
 
@@ -384,15 +388,21 @@ Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type,
   }
   else if(glom_type == Field::TYPE_IMAGE)
   {
-    //We assume that the text is the same format that we use in the document when saving examples.
+    //We assume that the text is the same (escaped text) format that we use in the document when saving examples.
     //(The SQL format).
     Gnome::Gda::Value result;
 
-    //libgda currently assumes that this buffer is an already-escaped string.
-    //TODO: Unescape the text when libgda has been fixed.
-    result.set(text.c_str(), text.size());
-    success = true;
-    return result;
+    size_t buffer_binary_length = 0;
+    guchar* buffer_binary =  Glom_PQunescapeBytea((const guchar*)text.c_str() /* must be null-terminated */, &buffer_binary_length); //freed by us later.
+    if(buffer_binary)
+    {
+      result.set(buffer_binary, buffer_binary_length);
+      success = true;
+
+      free(buffer_binary);
+
+      return result;
+    }
   }
 
   success = true;
@@ -400,12 +410,12 @@ Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type,
 
 }
 
-tm GlomConversions::parse_date(const Glib::ustring& text, bool& success)
+tm Conversions::parse_date(const Glib::ustring& text, bool& success)
 {
   return parse_date( text, std::locale("") /* the user's current locale */, success ); //Get the current locale.
 }
 
-tm GlomConversions::parse_date(const Glib::ustring& text, const std::locale& locale, bool& success)
+tm Conversions::parse_date(const Glib::ustring& text, const std::locale& locale, bool& success)
 {
   //return parse_tm(text, locale, 'x' /* date */);
 
@@ -515,7 +525,7 @@ tm GlomConversions::parse_date(const Glib::ustring& text, const std::locale& loc
 }
 
 
-tm GlomConversions::parse_time(const Glib::ustring& text, bool& success)
+tm Conversions::parse_time(const Glib::ustring& text, bool& success)
 {
   //return parse_time( text, std::locale("") /* the user's current locale */ ); //Get the current locale.
 
@@ -535,7 +545,7 @@ tm GlomConversions::parse_time(const Glib::ustring& text, bool& success)
 }
 
 
-tm GlomConversions::parse_time(const Glib::ustring& text, const std::locale& locale, bool& success)
+tm Conversions::parse_time(const Glib::ustring& text, const std::locale& locale, bool& success)
 {
   //The sequence of statements here seems to be very fragile. If you move things then it stops working.
 
@@ -594,7 +604,7 @@ tm GlomConversions::parse_time(const Glib::ustring& text, const std::locale& loc
 
 
 /* This requires an extension to the standard - time_get<>.get().:
-tm GlomConversions::parse_tm(const Glib::ustring& text, const std::locale& locale, char format)
+tm Conversions::parse_tm(const Glib::ustring& text, const std::locale& locale, char format)
 {
   //This is based on docs found here:
   //http://www.roguewave.com/support/docs/sourcepro/stdlibref/time-get.html
@@ -626,20 +636,21 @@ tm GlomConversions::parse_tm(const Glib::ustring& text, const std::locale& local
 }
 */
 
-bool GlomConversions::value_is_empty(const Gnome::Gda::Value& value)
+bool Conversions::value_is_empty(const Gnome::Gda::Value& value)
 {
+  if(value.is_null())
+    return true;
+
   switch(value.get_value_type())
   {
-    case(Gnome::Gda::VALUE_TYPE_NULL):
-      return true;
-    case(Gnome::Gda::VALUE_TYPE_STRING):
+    case(G_TYPE_STRING):
       return value.get_string().empty();
     default:
       return false; //None of the other types can be empty. (An empty numeric, date, or time type shows up as a null.
   }
 }
 
-Gnome::Gda::Value GlomConversions::get_empty_value(Field::glom_field_type field_type)
+Gnome::Gda::Value Conversions::get_empty_value(Field::glom_field_type field_type)
 {
   switch(field_type)
   {
@@ -650,7 +661,7 @@ Gnome::Gda::Value GlomConversions::get_empty_value(Field::glom_field_type field_
   }
 }
 
-Gnome::Gda::Value GlomConversions::get_example_value(Field::glom_field_type field_type)
+Gnome::Gda::Value Conversions::get_example_value(Field::glom_field_type field_type)
 {
   switch(field_type)
   {
@@ -824,9 +835,9 @@ Glom_PQunescapeBytea(const unsigned char *strtext, size_t *retbuflen)
   return tmpbuf;
 }
 
-Glib::ustring GlomConversions::get_escaped_binary_data(guint8* buffer, size_t buffer_size)
+Glib::ustring Conversions::get_escaped_binary_data(guint8* buffer, size_t buffer_size)
 {
-  //g_warning("GlomConversions::get_escaped_binary_data: debug: buffer ");
+  //g_warning("Conversions::get_escaped_binary_data: debug: buffer ");
   //for(int i = 0; i < 10; ++i)
   //  g_warning("%02X (%c), ", (guint8)buffer[i], buffer[i]);
 
@@ -866,26 +877,9 @@ Glib::ustring GlomConversions::get_escaped_binary_data(guint8* buffer, size_t bu
   return result;
 }
 
-Gnome::Gda::Value GlomConversions::parse_escaped_binary_data(const Glib::ustring& escaped_data)
+Gnome::Gda::Value Conversions::convert_value(const Gnome::Gda::Value& value, Field::glom_field_type target_glom_type)
 {
-  //Hopefully we don't need to use this because Gda does it for us when we read a part of a "SELECT" result into a Gnome::Value.
-  //TODO: Performance
-
-  Gnome::Gda::Value result;
-  size_t buffer_binary_length = 0;
-  guchar* buffer_binary =  Glom_PQunescapeBytea((guchar*)escaped_data.c_str(), &buffer_binary_length);
-  if(buffer_binary)
-  {
-    result.set(buffer_binary, buffer_binary_length);
-    free(buffer_binary);
-  }
-
-  return result;
-}
-
-Gnome::Gda::Value GlomConversions::convert_value(const Gnome::Gda::Value& value, Field::glom_field_type target_glom_type)
-{
-  const Field::glom_field_type source_glom_type = Field::get_glom_type_for_gda_type(value.get_value_type());
+  const Field::glom_field_type source_glom_type = Field::get_glom_type_for_gda_type(G_VALUE_TYPE(value.gobj()));
   if(source_glom_type == target_glom_type)
     return value; //No conversion necessary.
   else
@@ -898,4 +892,83 @@ Gnome::Gda::Value GlomConversions::convert_value(const Gnome::Gda::Value& value,
   return value;
 }
 
+Glib::RefPtr<Gdk::Pixbuf> Conversions::get_pixbuf_for_gda_value(const Gnome::Gda::Value& value)
+{
+  Glib::RefPtr<Gdk::Pixbuf> result;
+
+  if(value.get_value_type() == GDA_TYPE_BINARY)
+  {
+    glong buffer_binary_length = 0;
+    gconstpointer buffer_binary = value.get_binary(buffer_binary_length);
+
+    /* Note that this is regular binary data, not escaped text representing the binary data: */
+    if(buffer_binary && buffer_binary_length)
+    {
+      //typedef std::list<Gdk::PixbufFormat> type_list_formats;
+      //const type_list_formats formats = Gdk::Pixbuf::get_formats();
+      //std::cout << "Debug: Supported pixbuf formats:" << std::endl;
+      //for(type_list_formats::const_iterator iter = formats.begin(); iter != formats.end(); ++iter)
+      //{
+      //  std::cout << " name=" << iter->get_name() << ", writable=" << iter->is_writable() << std::endl;
+      //}
+
+      Glib::RefPtr<Gdk::PixbufLoader> refPixbufLoader;
+
+      // PixbufLoader::create() is broken in gtkmm before 2.6.something,
+      // so let's do this in C so it works with all 2.6 versions:
+      GError* error = 0;
+      GdkPixbufLoader* loader = gdk_pixbuf_loader_new_with_type(GLOM_IMAGE_FORMAT, &error);
+      if(!error)
+        refPixbufLoader = Glib::wrap(loader);
+      else
+      {
+        std::cerr << "ImageGlom::set_value(): Error while calling gdk_pixbuf_loader_new_with_type(): " << error->message << std::endl;
+	g_error_free(error);
+      }
+
+      /*
+      try
+      {
+        refPixbufLoader = Gdk::PixbufLoader::create(GLOM_IMAGE_FORMAT);
+        g_warning("debug a1");
+      }
+      catch(const Gdk::PixbufError& ex)
+      {
+        refPixbufLoader.clear();
+        g_warning("PixbufLoader::create failed: %s",ex.what().c_str());
+      }
+      */
+
+      if(refPixbufLoader)
+      {
+        try
+        {
+          guint8* puiData = (guint8*)buffer_binary;
+
+          //g_warning("ImageGlom::set_value(): debug: from db: ");
+          //for(int i = 0; i < 10; ++i)
+          //  g_warning("%02X (%c), ", (guint8)puiData[i], (char)puiData[i]);
+
+          refPixbufLoader->write(puiData, (glong)buffer_binary_length);
+
+          result = refPixbufLoader->get_pixbuf();
+
+          refPixbufLoader->close(); //This throws if write() threw, so it must be inside the try block.
+        }
+        catch(const Glib::Exception& ex)
+        {
+          g_warning("ImageGlom::set_value(): PixbufLoader::write() failed: %s", ex.what().c_str());
+        }
+      }
+
+      //TODO: load the image, using the mime type stored elsewhere.
+      //pixbuf = Gdk::Pixbuf::create_from_data(
+    }
+
+  }
+
+  return result;
+}
+
+} //namespace Glom
 
