@@ -26,6 +26,7 @@
 
 #include <glom/libglom/data_structure/layout/layoutitem_field.h>
 #include <libxml++/libxml++.h>
+#include <gdkmm/pixbuf.h>
 
 namespace Glom
 {
@@ -59,9 +60,11 @@ namespace Conversions
   Gnome::Gda::Value get_example_value(Field::glom_field_type field_type);
 
   Glib::ustring get_escaped_binary_data(guint8* buffer, size_t buffer_size);
-  Gnome::Gda::Value parse_escaped_binary_data(const Glib::ustring& escaped_data);
 
   Gnome::Gda::Value convert_value(const Gnome::Gda::Value& value, Field::glom_field_type target_glom_type);
+
+
+  Glib::RefPtr<Gdk::Pixbuf> get_pixbuf_for_gda_value(const Gnome::Gda::Value& value);
 }
 
 //Copied from Postgres's PQunescapeBytea() so I don't have the trouble of finding and linking to the 
